/***************************************************************************
 *   Copyright (C) 2007 by BogDan Vatra                                    *
 *   bogdan@licentia.eu                                                    *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#ifndef REPORTPAGE_H
#define REPORTPAGE_H
#include <QPaintDevice>
#include <QDomNode>
#include <QPrinter>

namespace Report
{

class Document;

class Page
{
public:
	enum SearchDirection {NextResult, PreviousResult };
public:
	~Page();
	void renderToDevice(QPaintDevice & device, QPainter * painter=0);
	QPrinter::PaperSize paperSize() const;
	QPrinter::Orientation paperOrientation() const;
	QSize pageSize() const;
	bool search(const QString &text, QRectF &rect, SearchDirection direction = NextResult, bool caseSensitive = false, int rectMargin = 5);

private:
	Page(QDomNode pageNode);
	friend class Document;

	void drawState(QPainter & p, QDomElement element);
	void drawRect(QPainter & p, QDomElement element);
	void drawLine(QPainter & p, QDomElement element);
	void drawEllipse(QPainter & p, QDomElement element);
	void drawPath(QPainter & p, QDomElement element);
	void drawPoint(QPainter & p, QDomElement element);
	void drawPolygon(QPainter & p, QDomElement element);
	void drawPixmap(QPainter & p, QDomElement element);
	void drawTextItem(QPainter & p, QDomElement element);
	void drawTiledPixmap(QPainter & p, QDomElement element);
	void drawImage(QPainter & p, QDomElement element);

private:
	QDomNode m_pageNode;
	QPrinter::PaperSize m_paperSize;
	QPrinter::Orientation m_paperOrientation;
	QDomElement m_searchNode;
	QString m_searchText;

};

}

#endif
