/***************************************************************************
 *   Copyright (C) 2007 by BogDan Vatra                                    *
 *   bogdan@licentia.eu                                                    *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include <QPrinter>
#include <QGraphicsSceneMouseEvent>
#include <QCursor>
#include <QKeyEvent>
#include <QDebug>

#include "pageinterface.h"
#include "bandinterface.h"

namespace Report
{

PageInterface::PageInterface(QObject *parent)
		: QGraphicsScene(parent), m_contextMenu(0), m_magnets(0), m_magnetArea(1)
{
	m_pageSize = A4;
	m_orientation = Portrait;
	setPageSize(A4);
	setOrientation(Portrait);
}

QRectF PageInterface::geometry()
{
	return m_geometry;
}

void PageInterface::setGeometry(QRectF rect)
{
	m_geometry = rect & sceneRect();
	emit geometryChanged(m_geometry);
}


PageInterface::PageSize PageInterface::pageSize()
{
	return m_pageSize;
}

void PageInterface::setPageSize(PageInterface::PageSize pageSize)
{
	m_pageSize = pageSize;
	updatePageRect();
	m_paperRect = sceneRect();
	setGeometry(QRect(sceneRect().x() + sceneRect().width()*0.12, sceneRect().y() + sceneRect().height()*0.03, sceneRect().width() - sceneRect().width()*0.15, sceneRect().height() - sceneRect().height()*0.06));
}

QRectF PageInterface::paperRect()
{
	return m_paperRect;
}

void PageInterface::setPaperRect(QRectF paperRect)
{
	m_paperRect = paperRect;
}


PageInterface::Orientation PageInterface::orientation()
{
	return m_orientation;
}

void PageInterface::setOrientation(PageInterface::Orientation orientation)
{
	m_orientation = orientation;
	updatePageRect();
	setGeometry(QRect(sceneRect().x() + sceneRect().width()*0.12, sceneRect().y() + sceneRect().height()*0.03, sceneRect().width() - sceneRect().width()*0.15, sceneRect().height() - sceneRect().height()*0.06));
}

int PageInterface::magnetArea()
{
	return m_magnetArea;
}

void PageInterface::setMagnetArea(int magnetArea)
{
	m_magnetArea = magnetArea;
}

void PageInterface::updatePageRect()
{
	QPrinter p;
	p.setOrientation((QPrinter::Orientation)orientation());
	p.setPageSize((QPrinter::PageSize)pageSize());
	setSceneRect(0, 0, p.paperRect(QPrinter::DevicePixel).width(), p.paperRect(QPrinter::DevicePixel).height());
}

void PageInterface::drawMagnets(ItemInterface* item)
{
	foreach(QGraphicsItem * it, m_gideLines)
		removeItem(it);

	m_gideLines.clear();

	if (!m_magnets)
		return;

	item->setToolTip(tr("Press Ctrl key to ingore magnets when drag the object"));

	/*
		QRadialGradient radialGrad(QPointF(m_magnetArea, m_magnetArea), m_magnetArea);
		radialGrad.setColorAt(0, Qt::red);
		radialGrad.setColorAt(0.5, Qt::blue);
		//radialGrad.setColorAt(1, Qt::red);
		radialGrad.setSpread(QGradient::PadSpread);
		QBrush brush(radialGrad);
	*/
	QPen p;

//	p.setWidth(1);//1+2*m_magnetArea);
	p.setColor(Qt::blue);
	p.setStyle(Qt::DotLine);
//	p.setBrush(brush);

	foreach(QGraphicsItem * it, items())
	{
		ItemInterface* ite = dynamic_cast<ItemInterface*>(it);

		if (!ite || !item->parentItem() || ite == item)
			continue;

		if ((m_magnets&Left) && abs(item->mapToScene(QPointF(0, 0)).x() - ite->mapToScene(QPointF(0, 0)).x()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(ite->mapToItem(item->parentItem(), QPointF(0, 0)).x(), item->geometry().y(), item->geometry().width(), item->geometry().height()));

			if (item->mapToScene(0, 0).y() < ite->mapToScene(0, ite->geometry().height()).y())
				m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), item->mapToScene(0, 0).y(), item->mapToScene(0, 0).x(), ite->mapToScene(0, ite->geometry().height()).y(), p));
			else
				m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), ite->mapToScene(0, 0).y(), item->mapToScene(0, 0).x(), item->mapToScene(0, item->geometry().height()).y(), p));
		}

		if ((m_magnets&Left) && abs(item->mapToScene(QPointF(0, 0)).x() - ite->mapToScene(QPointF(ite->geometry().width(), 0)).x()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(ite->mapToItem(item->parentItem(), QPointF(ite->geometry().width(), 0)).x(), item->geometry().y(), item->geometry().width(), item->geometry().height()));

			if (item->mapToScene(0, 0).y() < ite->mapToScene(0, ite->geometry().height()).y())
				m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), item->mapToScene(0, 0).y(), item->mapToScene(0, 0).x(), ite->mapToScene(0, ite->geometry().height()).y(), p));
			else
				m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), ite->mapToScene(0, 0).y(), item->mapToScene(0, 0).x(), item->mapToScene(0, item->geometry().height()).y(), p));
		}

		if ((m_magnets&Right) && abs(item->mapToScene(item->geometry().width(), 0).x() - ite->mapToScene(0, 0).x()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(ite->mapToItem(item->parentItem(), 0, 0).x() - item->geometry().width(), item->geometry().y(), item->geometry().width(), item->geometry().height()));

			if (item->mapToScene(item->geometry().width(), 0).y() < ite->mapToScene(ite->geometry().width(), ite->geometry().height()).y())
				m_gideLines.push_back(addLine(item->mapToScene(item->geometry().width(), 0).x(), item->mapToScene(item->geometry().width(), 0).y(), item->mapToScene(item->geometry().width(), 0).x(), ite->mapToScene(ite->geometry().width(), ite->geometry().height()).y(), p));
			else
				m_gideLines.push_back(addLine(item->mapToScene(item->geometry().width(), 0).x(), ite->mapToScene(ite->geometry().width(), 0).y(), item->mapToScene(item->geometry().width(), 0).x(), item->mapToScene(item->geometry().width(), item->geometry().height()).y(), p));
		}

		if ((m_magnets&Right) && abs(item->mapToScene(item->geometry().width(), 0).x() - ite->mapToScene(ite->geometry().width(), 0).x()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(ite->mapToItem(item->parentItem(), ite->geometry().width(), 0).x() - item->geometry().width(), item->geometry().y(), item->geometry().width(), item->geometry().height()));

			if (item->mapToScene(item->geometry().width(), 0).y() < ite->mapToScene(ite->geometry().width(), ite->geometry().height()).y())
				m_gideLines.push_back(addLine(item->mapToScene(item->geometry().width(), 0).x(), item->mapToScene(item->geometry().width(), 0).y(), ite->mapToScene(ite->geometry().width(), 0).x(), ite->mapToScene(ite->geometry().width(), ite->geometry().height()).y(), p));
			else
				m_gideLines.push_back(addLine(ite->mapToScene(ite->geometry().width(), 0).x(), ite->mapToScene(ite->geometry().width(), 0).y(), item->mapToScene(item->geometry().width(), 0).x(), item->mapToScene(item->geometry().width(), item->geometry().height()).y(), p));
		}

		if ((m_magnets&Top) && abs(item->mapToScene(QPointF(0, 0)).y() - ite->mapToScene(QPointF(0, 0)).y()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(item->geometry().x(), ite->mapToItem(item->parentItem(), QPointF(0, 0)).y(), item->geometry().width(), item->geometry().height()));
			m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), item->mapToScene(0, 0).y(), ite->mapToScene(0, 0).x(), item->mapToScene(0, 0).y(), p));
		}

		if ((m_magnets&Top) && abs(item->mapToScene(QPointF(0, 0)).y() - ite->mapToScene(QPointF(0, ite->geometry().height())).y()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(item->geometry().x(), ite->mapToItem(item->parentItem(), QPointF(0, ite->geometry().height())).y(), item->geometry().width(), item->geometry().height()));
			m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), item->mapToScene(0, 0).y(), ite->mapToScene(0, 0).x(), item->mapToScene(0, 0).y(), p));
		}

		if ((m_magnets&Bottom) && abs(item->mapToScene(QPointF(0, item->geometry().height())).y() - ite->mapToScene(QPointF(0, ite->geometry().height())).y()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(item->geometry().x(), ite->mapToItem(item->parentItem(), QPointF(0, ite->geometry().height())).y() - item->geometry().height(), item->geometry().width(), item->geometry().height()));
			m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), item->mapToScene(0, item->geometry().height()).y(), ite->mapToScene(0, 0).x(), item->mapToScene(0, item->geometry().height()).y(), p));
		}

		if ((m_magnets&Bottom) && abs(item->mapToScene(QPointF(0, item->geometry().height())).y() - ite->mapToScene(QPointF(0, 0)).y()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(item->geometry().x(), ite->mapToItem(item->parentItem(), QPointF(0, 0)).y() - item->geometry().height(), item->geometry().width(), item->geometry().height()));
			m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), item->mapToScene(0, item->geometry().height()).y(), ite->mapToScene(0, 0).x(), item->mapToScene(0, item->geometry().height()).y(), p));
		}

		if ((m_magnets&Horizontal) && abs(item->mapToScene(QPointF(0, item->geometry().height() / 2)).y() - ite->mapToScene(QPointF(0, ite->geometry().height() / 2)).y()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(item->geometry().x(), ite->mapToItem(item->parentItem(), QPointF(0, ite->geometry().height() / 2)).y() - item->geometry().height() / 2, item->geometry().width(), item->geometry().height()));
			m_gideLines.push_back(addLine(item->mapToScene(0, 0).x(), item->mapToScene(0, item->geometry().height() / 2).y(), ite->mapToScene(0, 0).x(), item->mapToScene(0, item->geometry().height() / 2).y(), p));
		}

		if ((m_magnets&Vertical) && abs(item->mapToScene(QPointF(item->geometry().width() / 2, 0)).x() - ite->mapToScene(QPointF(ite->geometry().width() / 2, 0)).x()) <= m_magnetArea)
		{
			item->setGeometry(QRectF(ite->mapToItem(item->parentItem(), QPointF(ite->geometry().width() / 2, 0)).x() - item->geometry().width() / 2, item->geometry().y(), item->geometry().width(), item->geometry().height()));
			m_gideLines.push_back(addLine(item->mapToScene(item->geometry().width() / 2, 0).x(), item->mapToScene(0, 0).y(), item->mapToScene(item->geometry().width() / 2, 0).x(), ite->mapToScene(0, ite->geometry().height()).y(), p));
		}

	}
}

void PageInterface::mouseMoveEvent(QGraphicsSceneMouseEvent * mouseEvent)
{
	Report::ItemInterface* item = 0;

	if (itemAt(mouseEvent->scenePos()))
		item = dynamic_cast<Report::ItemInterface*>(itemAt(mouseEvent->scenePos()));


	if (mouseEvent->buttons() == Qt::NoButton && item)
	{

		int pc = item->posibleResizeCurrsor(item->mapFromScene(mouseEvent->scenePos()));
		if (ItemInterface::Fixed != pc)
		{
			if ((pc&ItemInterface::ResizeTop && pc&ItemInterface::ResizeLeft) || (pc&ItemInterface::ResizeBottom && pc&ItemInterface::ResizeRight))
				item->setCursor(QCursor(Qt::SizeFDiagCursor));
			else
				if ((pc&ItemInterface::ResizeTop && pc&ItemInterface::ResizeRight) || (pc&ItemInterface::ResizeBottom && pc&ItemInterface::ResizeLeft))
					item->setCursor(QCursor(Qt::SizeBDiagCursor));
				else
					if ((pc&ItemInterface::ResizeTop) || (pc&ItemInterface::ResizeBottom))
						item->setCursor(QCursor(Qt::SizeVerCursor));
					else
						if ((pc&ItemInterface::ResizeRight) || (pc&ItemInterface::ResizeLeft))
							item->setCursor(QCursor(Qt::SizeHorCursor));
						else
							if (pc&ItemInterface::FixedPos)
								item->setCursor(QCursor(Qt::ArrowCursor));
		}
		else
			if (ItemInterface::Fixed == pc)
				item->setCursor(QCursor(Qt::OpenHandCursor));
	}

	QGraphicsScene::mouseMoveEvent(mouseEvent);

	if (mouseEvent->buttons() != Qt::LeftButton)
		return;

	item = 0;

	if (selectedItems().size())
		item = dynamic_cast<Report::ItemInterface*>(selectedItems()[0]);

	if (item && !(mouseEvent->modifiers()& Qt::ControlModifier))
		drawMagnets(item);
}

void PageInterface::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
	QGraphicsScene::mousePressEvent(event);
	if (event->buttons() == Qt::LeftButton && !itemAt(event->scenePos()))
		emit itemSelected(this, event->scenePos());
}

void PageInterface::mouseReleaseEvent(QGraphicsSceneMouseEvent * event)
{
	foreach(QGraphicsItem * item, m_gideLines)
		removeItem(item);
	m_gideLines.clear();
	QGraphicsScene::mouseReleaseEvent(event);
}

void PageInterface::keyReleaseEvent(QKeyEvent * keyEvent)
{
	foreach(QGraphicsItem * item, m_gideLines)
		removeItem(item);
	m_gideLines.clear();
	QGraphicsScene::keyReleaseEvent(keyEvent);
}

void PageInterface::keyPressEvent(QKeyEvent * keyEvent)
{
	if (keyEvent->modifiers()==Qt::NoModifier || (keyEvent->key()!=Qt::Key_Left && keyEvent->key()!=Qt::Key_Right && keyEvent->key()!=Qt::Key_Up && keyEvent->key()!=Qt::Key_Down))
	{
		QGraphicsScene::keyPressEvent(keyEvent);
		return;
	}

	if (keyEvent->modifiers() & Qt::AltModifier)
		foreach(QGraphicsItem * item, selectedItems())
		{
			if (dynamic_cast<Report::BandInterface*>(item))
				continue;

			if (dynamic_cast<Report::ItemInterface*>(item))
			{
				QRectF rect;
				switch(keyEvent->key())
				{
					case Qt::Key_Left:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x()-5,dynamic_cast<Report::ItemInterface*>(item)->geometry().y(),dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
					case Qt::Key_Right:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x()+5,dynamic_cast<Report::ItemInterface*>(item)->geometry().y(),dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
					case Qt::Key_Up:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x(),dynamic_cast<Report::ItemInterface*>(item)->geometry().y()-5,dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
					case Qt::Key_Down:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x(),dynamic_cast<Report::ItemInterface*>(item)->geometry().y()+5,dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
				}
				dynamic_cast<Report::ItemInterface*>(item)->setGeometry(rect);
				drawMagnets(dynamic_cast<Report::ItemInterface*>(item));
			}
		}

	if (keyEvent->modifiers() & Qt::ControlModifier)
		foreach(QGraphicsItem * item, selectedItems())
		{
			if (dynamic_cast<Report::BandInterface*>(item))
				continue;

			if (dynamic_cast<Report::ItemInterface*>(item))
			{
				QRectF rect;
				switch(keyEvent->key())
				{
					case Qt::Key_Left:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x()-1,dynamic_cast<Report::ItemInterface*>(item)->geometry().y(),dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
					case Qt::Key_Right:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x()+1,dynamic_cast<Report::ItemInterface*>(item)->geometry().y(),dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
					case Qt::Key_Up:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x(),dynamic_cast<Report::ItemInterface*>(item)->geometry().y()-1,dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
					case Qt::Key_Down:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x(),dynamic_cast<Report::ItemInterface*>(item)->geometry().y()+1,dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
				}
				dynamic_cast<Report::ItemInterface*>(item)->setGeometry(rect);
				drawMagnets(dynamic_cast<Report::ItemInterface*>(item));
			}
		}

	if (keyEvent->modifiers() & Qt::ShiftModifier)
		foreach(QGraphicsItem * item, selectedItems())
		{
			if (dynamic_cast<Report::ItemInterface*>(item))
			{
				QRectF rect;
				switch(keyEvent->key())
				{
					case Qt::Key_Left:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x(),dynamic_cast<Report::ItemInterface*>(item)->geometry().y(),dynamic_cast<Report::ItemInterface*>(item)->geometry().width()-1,dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
					case Qt::Key_Right:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x(),dynamic_cast<Report::ItemInterface*>(item)->geometry().y(),dynamic_cast<Report::ItemInterface*>(item)->geometry().width()+1,dynamic_cast<Report::ItemInterface*>(item)->geometry().height());
						break;
					case Qt::Key_Up:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x(),dynamic_cast<Report::ItemInterface*>(item)->geometry().y(),dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height()-1);
						break;
					case Qt::Key_Down:
						rect=QRectF(dynamic_cast<Report::ItemInterface*>(item)->geometry().x(),dynamic_cast<Report::ItemInterface*>(item)->geometry().y(),dynamic_cast<Report::ItemInterface*>(item)->geometry().width(),dynamic_cast<Report::ItemInterface*>(item)->geometry().height()+1);
						break;
				}
				dynamic_cast<Report::ItemInterface*>(item)->setGeometry(rect);
				drawMagnets(dynamic_cast<Report::ItemInterface*>(item));
			}
		}
}


void PageInterface::setContextMenu(QMenu * menu)
{
	m_contextMenu = menu;
}

void PageInterface::contextMenuEvent(QGraphicsSceneContextMenuEvent * contextMenuEvent)
{
	if (m_contextMenu)
		m_contextMenu->exec(contextMenuEvent->screenPos());
}

void PageInterface::setLeftMagnet(bool magneted)
{
	if (magneted)
		m_magnets |= Left;
	else
	{
		int a = 0xffff;
		a ^= Left;
		m_magnets &= a;
	}
}

void PageInterface::setRightMagnet(bool magneted)
{
	if (magneted)
		m_magnets |= Right;
	else
	{
		int a = 0xffff;
		a ^= Right;
		m_magnets &= a;
	}
}

void PageInterface::setTopMagnet(bool magneted)
{
	if (magneted)
		m_magnets |= Top;
	else
	{
		int a = 0xffff;
		a ^= Top;
		m_magnets &= a;
	}
}

void PageInterface::setBottomMagnet(bool magneted)
{
	if (magneted)
		m_magnets |= Bottom;
	else
	{
		int a = 0xffff;
		a ^= Bottom;
		m_magnets &= a;
	}
}

void PageInterface::setHorizontalMagnet(bool magneted)
{
	if (magneted)
		m_magnets |= Horizontal;
	else
	{
		int a = 0xffff;
		a ^= Horizontal;
		m_magnets &= a;
	}
}

void PageInterface::setVerticalMagnet(bool magneted)
{
	if (magneted)
		m_magnets |= Vertical;
	else
	{
		int a = 0xffff;
		a ^= Vertical;
		m_magnets &= a;
	}
}

void PageInterface::bandDestroyed(int type, int order)
{
	foreach(QObject *item, children())
		if (dynamic_cast<Report::BandInterface*>(item))
		{
			if (dynamic_cast<BandInterface*>(item)->bandType() == type && dynamic_cast<BandInterface*>(item)->order() > order)
				dynamic_cast<BandInterface*>(item)->setOrder(dynamic_cast<BandInterface*>(item)->order() - 1);
			if (dynamic_cast<BandInterface*>(item)->bandType() > type)
				dynamic_cast<BandInterface*>(item)->setOrder(dynamic_cast<BandInterface*>(item)->order());
		}
}

}
