/***************************************************************************
 *   Copyright (C) 2008 by BogDan Vatra                                    *
 *   bogdan@licentia.eu                                                    *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#include <QDebug>
#include "barcoderender.h"
#include "maxipng.h"

QImage renderBarecode(const QString & text, QString * errorStr, const QString & primaryMessage, int symbol, int frame, int height, int width, int security_level, int whitespace_width, int boder_width, int pdf417CodeWords, QRgb fgcolor, QRgb bgcolor)
{
	QImage img;

	struct zint_symbol *my_symbol;
	int error = 0;

	my_symbol = ZBarcode_Create();
	my_symbol->symbology=symbol;
	my_symbol->height=height;
	my_symbol->output_options=frame;
	my_symbol->whitespace_width=whitespace_width;
	my_symbol->border_width=boder_width;
	my_symbol->option_1=security_level;
	my_symbol->option_2=width;
	my_symbol->option_3=pdf417CodeWords;
	QByteArray bstr=text.toAscii();
	QByteArray pstr=primaryMessage.left(99).toAscii();
	strcpy(my_symbol->primary,pstr.data());
	error = ZBarcode_Encode(my_symbol, (unsigned char*)bstr.data());

	if (error != 0)
		if (errorStr)
			*errorStr=my_symbol->errtxt;

	if (error > WARN_INVALID_OPTION)
	{
		ZBarcode_Delete(my_symbol);
		return QImage();
	}

	if (my_symbol->symbology == BARCODE_MAXICODE)
	{
		long j;
		int i;
		int x_posn, y_posn, w_posn, h_posn;

		int gwidth = 300;
		int gheight = 300;



		QImage image(gwidth, gheight, QImage::Format_RGB32);

		for (j = 0;  j < gheight;  j++)
		{
			for (i = 0; i < gwidth; i ++)
				image.setPixel(i,j,bgcolor);

			if ((j % 18) < 12)
			{
				/* We are in a 'full' row of hexagons */
				for (i = 0; i < gwidth; i++)
				{
					x_posn = i / 10;
					y_posn = j / 18;
					y_posn *= 2;

					if (my_symbol->encoded_data[y_posn][x_posn] == '1')
					{
						/* A hexagon needs plotting here */
						w_posn = i % 10;
						h_posn = j % 18;

						if (hexagon[(h_posn * 10) + w_posn] == 1)
							image.setPixel(i,j,fgcolor);
					}
				}
			}

			if (((j + 9) % 18) < 12)
			{
				int ia, ja;
				ja = j - 9;
				/* We are in a 'reduced' row of hexagons */

				for (i = 5; i < (gwidth - 5); i++)
				{
					ia = i - 5;
					x_posn = ia / 10;
					y_posn = ja / 18;
					y_posn *= 2;
					y_posn++;

					if (my_symbol->encoded_data[y_posn][x_posn] == '1')
					{
						/* A hexagon needs plotting here */
						w_posn = ia % 10;
						h_posn = ja % 18;

						if (hexagon[(h_posn * 10) + w_posn] == 1)
							image.setPixel(i,j,fgcolor);
					}
				}
			}

			if ((j >= 103) && (j < 196))
				/* Central bullseye */
				for (i = 0; i < 93; i++)
					if (bullseye[(((j - 103) * 93) + i)] == 1)
						image.setPixel(99+i,j,fgcolor);
		}
		img=image;
	}
	else
	{
		long j;

		int i, k, offset, row, row_binding;
		float large_bar_height, preset_height;
		int large_bar_count;

		int gwidth = 2 * (my_symbol->width + (my_symbol->border_width * 2) + (my_symbol->whitespace_width * 2));
		row_binding = 0;

		if (my_symbol->height == 0)
			my_symbol->height = 50;

		large_bar_count = 0;

		preset_height = 0.0;

		for (i = 0; i < my_symbol->rows; i++)
		{
			preset_height += my_symbol->row_height[i];

			if (my_symbol->row_height[i] == 0)
				large_bar_count++;
		}

		large_bar_height = (my_symbol->height - preset_height) / large_bar_count;

		if (large_bar_count == 0)
			my_symbol->height = preset_height;

		int gheight = (2 * my_symbol->height) + (my_symbol->border_width * 4);

		if ((my_symbol->rows > 1) && ((my_symbol->output_options == BARCODE_BIND) || (my_symbol->output_options == BARCODE_BOX)))
			row_binding = 1;

		if (my_symbol->symbology == BARCODE_CODE16K)
			row_binding = 1;


		/* Pixel Plotting */
		offset = (2 * my_symbol->border_width) + (2 * my_symbol->whitespace_width);

		QImage image(gwidth, gheight, QImage::Format_RGB32);

		for (j = 0;  j < gheight;  j++)
		{
			/* top border */
			if (j < (2 * my_symbol->border_width))
			{
				if ((my_symbol->output_options == BARCODE_BOX) || (my_symbol->output_options == BARCODE_BIND))
					for (i = 0; i < gwidth; i++)
						image.setPixel(i,j,fgcolor);
				else
					for (i = 0; i < gwidth; i++)
						image.setPixel(i,j,bgcolor);
			}

			/* middle section */
			if ((j >= (2 * my_symbol->border_width)) && (j < (gheight - (2 * my_symbol->border_width))))
			{
				for (i = 0; i < gwidth; i++)
				{
					k = i;// (i / 3);

					/* left hand border */

					if (k < (2 * my_symbol->border_width))
					{
						if (my_symbol->output_options == BARCODE_BOX)
							image.setPixel(i,j,fgcolor);
						else
							image.setPixel(i,j,bgcolor);
					}

					/* left whitespace */
					if ((k >= (2 * my_symbol->border_width)) && (k < ((2 * my_symbol->border_width) + (2 * my_symbol->whitespace_width))))
						image.setPixel(i,j,bgcolor);

					/* the symbol area */
					if ((k >= ((2 * my_symbol->border_width) + (2 * my_symbol->whitespace_width))) && (k < (gwidth - ((2 * my_symbol->border_width) + (2 * my_symbol->whitespace_width)))))
					{
						float y_position, cumul;

						y_position = ((j / 2) - my_symbol->border_width);
						y_position += 1.0;
						row = 0;

						if (my_symbol->row_height[row] == 0)
						{
							cumul = large_bar_height;
						}
						else
						{
							cumul = my_symbol->row_height[row];
						}

						while (cumul < y_position)
						{
							row++;

							if (my_symbol->row_height[row] == 0)
							{
								cumul += large_bar_height;
							}
							else
							{
								cumul += my_symbol->row_height[row];
							}
						}

						switch (my_symbol->encoded_data[row][(k - offset) / 2])
						{

							case '1':
								image.setPixel(i,j,fgcolor);
								break;

							case 'R':
								image.setPixel(i,j,qRgb(0xff,0x00,0x00));
								break;

							case 'G':
								image.setPixel(i,j,qRgb(0x00,0xff,0x00));
								break;

							case 'B':
								image.setPixel(i,j,qRgb(0x00,0x00,0xff));
								break;

							case 'C':
								image.setPixel(i,j,qRgb(0x00,0xff,0xff));
								break;

							case 'M':
								image.setPixel(i,j,qRgb(0xff,0x00,0xff));
								break;

							case 'Y':
								image.setPixel(i,j,qRgb(0xff,0xff,0x00));
								break;

							default:
								image.setPixel(i,j,bgcolor);
								break;
						}

						/* row binding */
						if (row_binding)
						{
							/* Only valid if all rows are the same height */
							float total_height, equal_height;

							total_height = (gheight / 2) - my_symbol->border_width;
							equal_height = total_height / my_symbol->rows;

							if ((y_position > ((equal_height * row) - 1.0)) && (y_position < ((equal_height * row) + 1.0)))
								image.setPixel(i,j,fgcolor);

							if ((y_position > ((equal_height * (row + 1)) - 1.0)) && (y_position < ((equal_height * (row + 1)) + 1.0)))
								image.setPixel(i,j,fgcolor);
						}

					}

					/* right whitespace */
					if ((k >= (gwidth - ((2 *my_symbol->border_width) + (2 * my_symbol->whitespace_width)))) && (k < (gwidth - (2 * my_symbol->border_width))))
						image.setPixel(i,j,bgcolor);

					/* right hand border */
					if (k >= (gwidth - (2 * my_symbol->border_width)))
					{
						if (my_symbol->output_options == BARCODE_BOX)
							image.setPixel(i,j,fgcolor);
						else
							image.setPixel(i,j,bgcolor);
					}
				}
			}

			/* bottom border */
			if (j >= (gheight - (2 * my_symbol->border_width)))
			{
				if ((my_symbol->output_options == BARCODE_BOX) || (my_symbol->output_options == BARCODE_BIND))
					for (i = 0; i < gwidth; i++)
						image.setPixel(i,j,fgcolor);
				else
					for (i = 0; i < (gwidth * 3); i+=3)
						image.setPixel(i,j,bgcolor);
			}
		}
		img=image;
	}

	ZBarcode_Delete(my_symbol);

	return img;
}

