/***************************************************************************
 *   Copyright (C) 2008 by BogDan Vatra                                    *
 *   bogdan@licentia.eu                                                    *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
#include <QtCore>
#include <QFileDialog>
#include <QMessageBox>
#include <QDomDocument>
#include <errno.h>

#include "ods.h"
#include "zip.h"


inline void initMyResource()
{
	Q_INIT_RESOURCE(odsResource);
}

Ods::Ods(QObject *parent)
 : Report::ExportInterface(parent)
{
	initMyResource();
}


QString Ods::exportFormat()
{
	return tr("OASIS Spreadsheet *.ods");
};

void Ods::execExport(QDomNode & exportNode)
{
	QString fileName=QFileDialog::getSaveFileName(0,tr("Export"),QDir::homePath()+"/export.ods", tr("ODS (*.ods)"));
	if (!fileName.length())
		return;
	QDomDocument content;
	QFile file(":/content.xml");
	if (!file.open(QIODevice::ReadOnly))
		return;
	content.setContent(&file);
	file.close();

	file.setFileName(":/manifest.xml");
	if (!file.open(QIODevice::ReadOnly))
		return;
	QByteArray manifest=file.readAll();
	file.close();

	file.setFileName(":/meta.xml");
	if (!file.open(QIODevice::ReadOnly))
		return;
	QByteArray meta=file.readAll();
	file.close();

	QDomElement spreadRoot=content.documentElement().firstChildElement("office:body").firstChildElement("office:spreadsheet");
	if (spreadRoot.isNull())
		return;

	QDomElement qryEl=exportNode.firstChildElement("query");
	while(!qryEl.isNull())
	{
		QDomElement se=content.createElement("table:table");

		se.setAttribute("table:name",qryEl.attribute("name","query"));
		se.setAttribute("table:style-name","ta1");

		QDomNode rowNode=qryEl.firstChild();
		while(!rowNode.isNull())
		{
			QDomElement tr=content.createElement("table:table-row");

			QString line;
			QDomElement field=rowNode.firstChildElement();
			while(!field.isNull())
			{
				QDomElement tf=content.createElement("table:table-cell");
				if (field.attribute("type","string")=="float")
				{
					tf.setAttribute("office:value-type", "float");
					tf.setAttribute("office:value", field.text());
				}
				else
					tf.setAttribute("office:value-type", "string");

				QDomElement tfp=content.createElement("text:p");
				tfp.appendChild(content.createTextNode(field.text()));
				tf.appendChild(tfp);
				tr.appendChild(tf);
				field=field.nextSiblingElement();
			}
			se.appendChild(tr);
			rowNode=rowNode.nextSibling();
		}
		spreadRoot.appendChild(se);
		qryEl=qryEl.nextSiblingElement();
	}

	QByteArray cont=content.toString(2).toAscii();

	int errorp;
	char errorstr[512];
	zip * z=zip_open(fileName.toAscii(), ZIP_CREATE, &errorp);
	if (!z)
	{
		zip_error_to_str(errorstr,512,errorp,errno);
		qDebug()<<(char*)errorstr;
		QMessageBox::critical(0,tr("Export"),errorstr,QMessageBox::Ok);
		return;
	}


	zip_add_dir(z,"META-INF");

	zip_source * zm=zip_source_buffer(z, meta.data(), meta.length(), 0);
	zip_add(z,"meta.xml",zm);

	zip_source * zs=zip_source_buffer(z, manifest.data(), manifest.length(), 0);
	zip_add(z,"META-INF/manifest.xml",zs);

	zip_source * zc=zip_source_buffer(z, cont.data(), cont.length(), 0);
	zip_add(z,"content.xml",zc);

	zip_close(z);

	QMessageBox::information(0,tr("Export"),tr("Export done"),QMessageBox::Ok);
}

Ods::~Ods()
{
}

Q_EXPORT_PLUGIN2(ods, Ods)
