/***************************************************************************
 *   Copyright (C) 2008 by BogDan Vatra                                    *
 *   bogdan@licentia.eu                                                    *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include <QtCore>
#include <QBrush>
#include <QPainter>
#include <QStyleOptionGraphicsItem>
#include <QFontMetricsF>
#include <QDebug>

#include "barcode.h"
#include "barcoderender.h"

inline void initMyResource()
{
	Q_INIT_RESOURCE(barcode);
}

Barcode::Barcode(QGraphicsItem* parent, QObject* parentObject) : ItemInterface(parent, parentObject)
{
	m_barcodeType = (Barcode::BarcodeTypes)BARCODE_CODE93;
	m_paintType = RESIZE_ITEM_TO_BARCODE;
	m_frameType = NO_FRAME;
	m_barcodeHeight = 50;
	m_barcodeWhitespaceWidth = 0;;
	m_barcodeBoderWidth = 0;
	m_barcodeForegroundColor = qRgb(0, 0, 0);
	m_barcodeBackgroundColor = qRgb(0xff, 0xff, 0xff);
	m_barcodeWidth = 0;
	m_barcodeSecurityLevel = -1;
	m_script = "05072006";
	m_testText = "05072006";
	m_drawTextType = NO_TEXT;
	m_pdf417Max = 928;
	initMyResource();
	setWidth(75);
}

Barcode::BarcodeTypes Barcode::barcodeType()
{
	return m_barcodeType;
}
void Barcode::setBarcodeType(BarcodeTypes barcodeType)
{
	m_barcodeType = barcodeType;
	update();
}

Barcode::PaintTypes Barcode::paintType()
{
	return m_paintType;
}
void Barcode::setPaintType(PaintTypes paintType)
{
	m_paintType = paintType;
	update();
}

Barcode::FrameTypes Barcode::frameType()
{
	return m_frameType;
}

void Barcode::setFrameType(FrameTypes frameType)
{
	m_frameType = frameType;
	update();
}

int Barcode::barcodeHeight()
{
	return m_barcodeHeight;
}
void Barcode::setBarcodeHeight(int barcodeHeight)
{
	m_barcodeHeight = barcodeHeight;
	update();
}

int Barcode::barcodeWhitespaceWidth()
{
	return m_barcodeWhitespaceWidth;
}
void Barcode::setBarcodeWhitespaceWidth(int barcodeWhitespaceWidth)
{
	m_barcodeWhitespaceWidth = barcodeWhitespaceWidth;
	update();
}

int Barcode::barcodeBoderWidth()
{
	return m_barcodeBoderWidth;
}
void Barcode::setBarcodeBoderWidth(int barcodeBoderWidth)
{
	m_barcodeBoderWidth = barcodeBoderWidth;
	update();
}

QColor Barcode::barcodeForegroundColor()
{
	return m_barcodeForegroundColor;
}
void Barcode::setBarcodeForegroundColor(QColor barcodeForegroundColor)
{
	m_barcodeForegroundColor = barcodeForegroundColor;
	update();
}

QColor Barcode::barcodeBackgroundColor()
{
	return m_barcodeBackgroundColor;
}
void Barcode::setBarcodeBackgroundColor(QColor barcodeBackgroundColor)
{
	m_barcodeBackgroundColor = barcodeBackgroundColor;
	update();
}

int Barcode::barcodeWidth()
{
	return m_barcodeWidth;
}
void Barcode::setBarcodeWidth(int barcodeWidth)
{
	m_barcodeWidth = barcodeWidth;
	update();
}

int Barcode::barcodeSecurityLevel()
{
	return m_barcodeSecurityLevel;
}
void Barcode::setBarcodeSecurityLevel(int barcodeSecurityLevel)
{
	m_barcodeSecurityLevel = barcodeSecurityLevel;
	update();
}

int Barcode::pdf417Max()
{
	return m_pdf417Max;
}
void Barcode::setPdf417Max(int pdf417Max)
{
	m_pdf417Max=pdf417Max;
	if (m_pdf417Max<928)
		m_pdf417Max=928;
	if (m_pdf417Max>1900)
		m_pdf417Max=1900;
}

Barcode::DrawTextTypes Barcode::drawTextType()
{
	return m_drawTextType;
}
void Barcode::setDrawTextType(DrawTextTypes drawTextType)
{
	m_drawTextType = drawTextType;
	update();
}

const QString & Barcode::primaryMessage()
{
	return m_primaryMessage;
}
void Barcode::setPrimaryMessage(const QString &primaryMessage)
{
	m_primaryMessage=primaryMessage;
	update();
}

QString Barcode::script()
{
	return m_script;
}

void Barcode::setScript(const QString &script)
{
	m_script = script;
	update();
}

QString Barcode::testText()
{
	return m_testText;
}
void Barcode::setTestText(const QString & testText)
{
	m_testText=testText;
	update();
}

QRectF Barcode::boundingRect() const
{
	return QRectF(0, 0, width(), height());
}

void Barcode::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * /*widget*/)
{
	QString error;
	QString m_text;
	if (option->type == QStyleOption::SO_GraphicsItem)
		m_text=m_testText;
	else
		m_text=scriptEngine()->evaluate(m_script).toString();
	qDebug()<<"after";
	QImage barcode = renderBarecode(m_text, &error, m_primaryMessage, m_barcodeType, m_frameType, m_barcodeHeight, m_barcodeWidth, m_barcodeSecurityLevel, m_barcodeWhitespaceWidth, m_barcodeBoderWidth, m_pdf417Max,  m_barcodeForegroundColor.rgba(), m_barcodeBackgroundColor.rgba());
	qDebug()<<"before";

	if (option->type != QStyleOption::SO_GraphicsItem)
		emit beforePrint(this);

	QRectF rect = (option->type == QStyleOption::SO_GraphicsItem) ? boundingRect() : option->exposedRect;

	if (option->type == QStyleOption::SO_GraphicsItem)
		drawSelection(painter, boundingRect());

	setupPainter(painter);

	adjustRect(rect);

	qreal texth = 0;
	qreal sw=0;
	qreal sh=0;

	if (m_drawTextType == DRAW_ABOVE || m_drawTextType == DRAW_BELOW)
		texth = QFontMetricsF(painter->font()).height();

	if (barcode.isNull())
		painter->drawText(rect, Qt::AlignCenter, error);
	else
	{
		QImage img;
		QPointF point = rect.topLeft();
		int cx=0,cy=0,cw=barcode.width(),ch=barcode.height();
		switch (m_paintType)
		{
			case RESIZE_BARCODE_TO_ITEM_KEEP_ASPECT_RATIO:
				img = barcode.scaled(rect.width(), rect.height() - texth, Qt::KeepAspectRatio, Qt::SmoothTransformation);
				break;
			case RESIZE_BARCODE_TO_ITEM_IGNORE_ASPECT_RATIO:
				img = barcode.scaled(rect.width(), rect.height() - texth, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
				break;
			case CENTER_BARCODE_TO_ITEM:
				point.setX(point.x()+(rect.width()-barcode.width())/2);
				point.setY(point.y()+(rect.height()-barcode.height()-texth)/2);
				if (point.x()<0)
				{
					cx=abs(point.x());
					cw-=2*cx;
					point.setX(0);
				}
				if (point.y()<0)
				{
					cy=abs(point.y());
					ch-=2*cy;
					point.setY(0);
				}
				img=barcode.copy(cx,cy,cw,ch);
				break;
			case RESIZE_ITEM_TO_BARCODE:
				img=barcode;
				sw=img.width()-width();
				sh=img.height()-(height()-texth);
				break;
		}
		if (m_drawTextType == DRAW_ABOVE)
			point.setY(point.y()+texth);;
		painter->drawImage(point, img);
	}

	switch (m_drawTextType)
	{
		case DRAW_TOP:
			painter->drawText(rect, Qt::AlignTop| Qt::AlignHCenter, m_text);
			break;
		case DRAW_BOTTOM:
			painter->drawText(rect, Qt::AlignBottom| Qt::AlignHCenter, m_text);
			break;
		case DRAW_ABOVE:
			painter->drawText(rect, Qt::AlignTop| Qt::AlignHCenter, m_text);
			break;
		case DRAW_BELOW:
			painter->drawText(rect, Qt::AlignBottom| Qt::AlignHCenter, m_text);
			break;
		default:
			break;
	}
	if (option->type != QStyleOption::SO_GraphicsItem)
		emit afterPrint(this);

	if (sw || sh)
	{
		setWidth(width()+sw);
		setHeight(height()+sh);
	}
}

QIcon Barcode::toolBoxIcon()
{
	return QIcon(":/barcode.png");
};

QString Barcode::toolBoxText()
{
	return tr("Barcode");
}

QString Barcode::toolBoxGroup()
{
	return tr("Shapes");
}

QObject * Barcode::createInstance(QGraphicsItem* parent, QObject* parentObject)
{
	return new Barcode(parent, parentObject);
}

Q_EXPORT_PLUGIN2(barcode, Barcode)
